#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <yubihsm.h>

#define DEFAULT_CONNECTOR_URL "http://127.0.0.1:12345"

const uint8_t password[] = "password";
const uint8_t data[] = "sudo make me a sandwich";

int main(void) {
  yh_connector *connector = NULL;
  uint8_t context[YH_CONTEXT_LEN];
  yh_rc yrc = YHR_GENERIC_ERROR;

  yrc = yh_init();
  if (yrc != YHR_SUCCESS) {
    fprintf(stderr, "unable to initialize yubihsm: %s\n", yh_strerror(yrc));
    exit(EXIT_FAILURE);
  }

  yrc = yh_init_connector(DEFAULT_CONNECTOR_URL, &connector);
  assert(yrc == YHR_SUCCESS);

  yrc = yh_connect_best(&connector, 1, NULL);
  assert(yrc == YHR_SUCCESS);

  char *received_url;
  yrc = yh_get_connector_address(connector, &received_url);
  assert(yrc == YHR_SUCCESS);

  yh_set_verbosity(YH_VERB_ALL);

  printf("Send a plain (unencrypted, unauthenticated) echo command\n");

  uint16_t data_len = sizeof(data) - 1;
  uint8_t response[sizeof(data)] = {0};
  size_t response_len = sizeof(response);
  yh_cmd response_cmd;
  yrc = yh_send_plain_msg(connector, YHC_ECHO, data, data_len, &response_cmd,
                          response, &response_len);
  if (yrc != YHR_SUCCESS) {
    fprintf(stderr, "Failed to send ECHO command): %s\n", yh_strerror(yrc));
    exit(EXIT_FAILURE);
  }

  if (response_cmd == YHC_ERROR) {
    fprintf(stderr, "Unable to get echo data: %s (%x)\n",
            yh_strerror(response[0]), response[0]);
    exit(EXIT_FAILURE);
  }

  printf("Response (%zu bytes): \"%s\"\n", response_len, response);

  yh_session *session = NULL;
  uint16_t keyset = 1;
  yrc = yh_create_session_derived(connector, keyset, password, sizeof(password),
                                  false, context, sizeof(context), &session);
  assert(yrc == YHR_SUCCESS);

  yrc = yh_authenticate_session(session, context, sizeof(context));
  assert(yrc == YHR_SUCCESS);

  uint8_t session_id;
  yrc = yh_get_session_id(session, &session_id);
  assert(yrc == YHR_SUCCESS);

  printf("Successfully established session %02d\n", session_id);

  printf("Send a secure echo command\n");

  uint8_t response2[sizeof(data)] = {0};
  size_t response2_len = sizeof(response);
  yh_cmd response2_cmd;
  yrc = yh_send_secure_msg(session, YHC_ECHO, data, data_len, &response2_cmd,
                           response2, &response2_len);
  if (yrc != YHR_SUCCESS) {
    fprintf(stderr, "Failed to send ECHO command: %s\n", yh_strerror(yrc));
    exit(EXIT_FAILURE);
  }

  if (response_cmd == YHC_ERROR) {
    fprintf(stderr, "Unable to get echo data: %s (%x)\n",
            yh_strerror(response[0]), response[0]);
    exit(EXIT_FAILURE);
  }

  printf("Response (%zu bytes): \"%s\"\n", response_len, response);

  assert(response_len == response2_len);
  assert(memcmp(response, response2, response_len) == 0);

  yrc = yh_util_close_session(session);
  assert(yrc == YHR_SUCCESS);

  yrc = yh_destroy_session(&session);
  assert(yrc == YHR_SUCCESS);

  yh_disconnect(connector);
  assert(yrc == YHR_SUCCESS);

  yrc = yh_exit();
  assert(yrc == YHR_SUCCESS);

  return 0;
}
