#include <openssl/bn.h>
#include <openssl/evp.h>
#include <stdbool.h>
#include <stdint.h>

#include <yubihsm.h>

typedef enum {
  _base64,
  _binary,
  _hex,
} format_t;

bool set_component(uint8_t *in_ptr, const BIGNUM *bn, int32_t element_len);
bool read_private_key(uint8_t *buf, size_t len, yh_algorithm *algo,
                      uint8_t *bytes, size_t *bytes_len, bool internal_repr);
void format_digest(uint8_t *digest, char *str, uint16_t len);
int algo2nid(yh_algorithm algo);
bool algo2type(yh_algorithm algorithm, yh_object_type *type);
void parse_NID(uint8_t *data, uint16_t data_len, const EVP_MD **md_type,
               int *digestinfo_len);
bool read_file(FILE *fp, uint8_t *buf, size_t *buf_len);
bool write_file(const uint8_t *buf, size_t buf_len, FILE *fp, format_t format);
bool read_ed25519_key(uint8_t *in, size_t in_len, uint8_t *out,
                      size_t *out_len);
bool write_ed25519_key(uint8_t *buf, size_t buf_len, FILE *fp, bool b64_encode);

bool base64_decode(const char *in, uint8_t *out, size_t *len);
bool hex_decode(const char *in, uint8_t *out, size_t *len);

bool split_hmac_key(yh_algorithm algorithm, uint8_t *in, size_t in_len,
                    uint8_t *out, size_t *out_len);
